/**
     *  Onelog extension
 */
var olExtension = {
    Version: '',
    Functions: {
        Sleep: function (ms) {
            try {
                var dt = new Date();
                dt.setTime(dt.getTime() + ms);
                while (new Date().getTime() < dt.getTime()) {
                }
            } catch (e) {
                var alertCatch_ = new olFunctions.AlertContent('Sleep', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertCatch_);
            }
        },
        UniqueArray: function (array) {
            var uniqueArray_ = array.concat();
            for (var i = 0; i < uniqueArray_.length; ++i) {
                for (var j = i + 1; j < uniqueArray_.length; ++j) {
                    if (uniqueArray_[i] === uniqueArray_[j])
                        uniqueArray_.splice(j--, 1);
                }
            }
            return uniqueArray_;
        },
        fixIEObjectToArray: function (objectToTransformToArray, destinationArray) {
            try {
                for (var c = 0; c < objectToTransformToArray.length; c++) {
                    destinationArray.push(olFunctions.Clone(objectToTransformToArray[c]));
                }
                return destinationArray;
            } catch (e) {
                var alertErrorContent_ = new olFunctions.AlertContent('fixIEObjectToArray', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
            }
        },
        CloseTabFunction: function (targetTab, delay) {
            try {
                setTimeout(function () {
                    var alertErrorContent_;
                    try {
                        targetTab.close();
                    } catch (e) {
                        alertErrorContent_ = new olFunctions.AlertContent('CloseTabFunction failed to close tab', e.message + '\n' + e.stack);
                        olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                    }
                }, delay);
            } catch (e) {
                var alertCatch_ = new olFunctions.AlertContent('CloseTab', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertCatch_);
            }
        },
        HandleCookiesArray: function () {
            try {
                if (olFunctions.IsFilledArray(olData.CookiesInfoArray.TabMembers)) {
                    var allDomainsToBeRemoved_ = [];
                    for (var i in olData.CookiesInfoArray.TabMembers) {
                        // Get all domains which cookies should be removed
                        allDomainsToBeRemoved_ = olExtension.Functions.UniqueArray(allDomainsToBeRemoved_.concat(olData.CookiesInfoArray.TabMembers[i].Domains));
                    }
                    olExtension.Functions.RemoveDomainCookies(allDomainsToBeRemoved_);

                    olData.CookiesInfoArray.TabMembers = [];
                    olData.CookiesInfoArray.TabMembersLocked = false;
                }
            } catch (e) {
                var alertErrorContent_ = new olFunctions.AlertContent('HandleCookiesArray', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
            }
        },
        RemoveDomainCookies: function (resourceDomains) {
            try {
                var removeCookies_ = {};
                if (!chrome.cookies) {
                    chrome.cookies = chrome.experimental.cookies;
                }

                chrome.cookies.getAll({}, function (cookies) {

                    if (cookies) {
                        for (var i in cookies) {
                            var currentCookie_ = cookies[i];
                            if (!currentCookie_.expirationDate) {
                                // delete cookies that don't have expiration date only
                                for (var j in resourceDomains) {
                                    var currentResourceDomain_ = resourceDomains[j];
                                    if (("." + currentResourceDomain_).indexOf(currentCookie_.domain) != -1) {
                                        var key_ = currentCookie_.name + currentCookie_.domain + currentCookie_.hostOnly + currentCookie_.path + currentCookie_.secure + currentCookie_.httpOnly + currentCookie_.session + currentCookie_.storeId;
                                        removeCookies_[key_] = currentCookie_;
                                        break;
                                    }
                                }
                            }
                        }
                        for (var k in removeCookies_) {
                            var removeCookie_ = removeCookies_[k];
                            var url_ = "http" + (removeCookie_.secure ? "s" : "") + "://" + removeCookie_.domain + removeCookie_.path;
                            var name_ = removeCookie_.name;
                            chrome.cookies.remove({
                                "url": url_,
                                "name": name_
                            });
                        }
                    } else {
                        var alertErrorContent_ = new olFunctions.AlertContent('RemoveDomainCookies ', ' NO COOKIES!');
                        olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                    }
                });
            } catch (e) {
                var alertErrorContent_ = new olFunctions.AlertContent('RemoveDomainCookies', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
            }
        },
        RemoveCookies: function (tabMember) {
            try {
                var tabMembers_ = olExtension._internal.TabMember.Filter(tabMember.WindowHandle, null, null, tabMember.ApplicationsSessionId);
                if (tabMembers_.length === 0) {
                    olData.CookiesInfoArray.TabMembers.push(tabMember);
                }
            } catch (e) {
                olData.CookiesInfoArray.TabMembers.push(tabMember);
                var alertErrorContent_ = new olFunctions.AlertContent('RemoveCookies', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
            }
        }
    },
    _internal: {
        TabMember: {
            _internal: {
                FreeTabMembers: function () {
                    try {
                        // var alertContent_ = new olFunctions.AlertContent('FreeTabMembers', 'Extension function');
                        // olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                        olData.TabInfoArray.TabMembersLocked = false;
                    } catch (e) {
                        var alertErrorContent_ = new olFunctions.AlertContent('FreeTabMembers', e.message + '\n' + e.stack);
                        olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                    }
                },
                LockTabMembers: function () {
                    try {
                        // var alertContent_ = new olFunctions.AlertContent('LockTabMembers', 'Extension function');
                        // olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                        while (olData.TabInfoArray.TabMembersLocked) {
                            olExtension.Functions.Sleep(olOptions.General.Extension.TabInfoArraySleepInterval());
                        }
                        olData.TabInfoArray.TabMembersLocked = true;
                    } catch (e) {
                        var alertErrorContent_ = new olFunctions.AlertContent('LockTabMembers', e.message + '\n' + e.stack);
                        olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                    }
                },
                FreeLogoutTabMembers: function () {
                    try {
                        // var alertContent_ = new olFunctions.AlertContent('FreeLogoutTabMembers', 'Extension function');
                        // olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                        olData.LogoutInfoArray.TabMembersLocked = false;
                    } catch (e) {
                        var alertErrorContent_ = new olFunctions.AlertContent('FreeLogoutTabMembers', e.message + '\n' + e.stack);
                        olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                    }
                },
                LockLogoutTabMembers: function () {
                    try {
                        // var alertContent_ = new olFunctions.AlertContent('LockLogoutTabMembers', 'Extension function');
                        // olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                        while (olData.LogoutInfoArray.TabMembersLocked) {
                            olExtension.Functions.Sleep(olOptions.General.Extension.TabInfoArraySleepInterval());
                        }
                        olData.LogoutInfoArray.TabMembersLocked = true;
                    } catch (e) {
                        var alertErrorContent_ = new olFunctions.AlertContent('LockLogoutTabMembers', e.message + '\n' + e.stack);
                        olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                    }
                }
            },
            Add: function (tabMember) {
                try {
                    // var alertContent_ = new olFunctions.AlertContent('Add (tabMember)', 'Extension function');
                    // olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                    olExtension._internal.TabMember._internal.LockTabMembers();
                    var tabMembers_ = ol$Functions.grep(olData.TabInfoArray.TabMembers, function (member) {
                        return (member.Id == tabMember.Id);
                    });
                    if (!olFunctions.IsFilledArray(tabMembers_)) {
                        olData.TabInfoArray.TabMembers.push(tabMember);
                        olData._DebugStatisticsTabInfoArray.push(tabMember.TabId)
                    }

                    olExtension._internal.TabMember._internal.FreeTabMembers();
                } catch (e) {
                    var alertErrorContent_ = new olFunctions.AlertContent('Add', e.message + '\n' + e.stack);
                    olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                }
            },
            Create: function (tabId, url, framesArray) {
                try {
                    var alertContent_ = new olFunctions.AlertContent('Create (tabMember)', 'Extension function');
                    olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                    var clippedUrl = url.substring(0, 2000)
                    var tabMember_ = new olExtension.DataConstructors.TabMember(tabId, clippedUrl, framesArray);
                    olExtension._internal.TabMember.Add(tabMember_);

                    olWrap.storage.asyncSetOlData(olData);
                    return tabMember_;
                } catch (e) {
                    var alertErrorContent_ = new olFunctions.AlertContent('Create', e.message + '\n' + e.stack);
                    olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                }
            },
            ChangeApplicationIdAndStartTime: function (fromAppId, toAppId, bnrStartTime, tmPauseDuration) {
                try {
                    // var alertContent_ = new olFunctions.AlertContent('ChangeApplicationIdAndStartTime', 'Extension function');
                    // olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                    olExtension._internal.TabMember._internal.LockTabMembers();
                    var tabMembers_ = ol$Functions.grep(olData.TabInfoArray.TabMembers, function (member) {
                        try {
                            return (member.PageInfo.Response.BeforeNavigateResponses[0].ApplicationsSessionId == fromAppId);
                        } catch (e) {
                            var alertErrorContent_ = new olFunctions.AlertContent('ChangeApplicationIdAndStartTime GREP failed', e.message + '\n' + e.stack);
                            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                        }
                    });
                    if (olFunctions.IsFilledArray(tabMembers_)) {
                        for (var i = 0; i < tabMembers_.length; i++) {
                            tabMembers_[i].PageInfo.Response.BeforeNavigateResponses[0].ApplicationsSessionId = toAppId;
                            tabMembers_[i].ApplicationsSessionId = toAppId;
                            tabMembers_[i].PageInfo.Response.BeforeNavigateResponses[0].StartTime = bnrStartTime;
                            if (tmPauseDuration) {
                                tabMembers_[i].PageInfo.Response.BeforeNavigateResponses[0].PauseDuration = tmPauseDuration;
                            } else {
                                tabMembers_[i].PageInfo.Response.BeforeNavigateResponses[0].PauseDuration = 0;
                            }
                        }
                    }
                    olExtension._internal.TabMember._internal.FreeTabMembers();
                } catch (e) {
                    var alertErrorContent_ = new olFunctions.AlertContent('ChangeApplicationIdAndStartTime', e.message + '\n' + e.stack);
                    olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                }
            },
            _getLocked: false,
            Filter: function (windowHandle, tabId, url, applicationsSessionId) {
                try {
                    // var alertContent_ = new olFunctions.AlertContent('Filter (tabMember)', 'Extension function');
                    // olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                    if (!olExtension._internal.TabMember._getLocked) {
                        olExtension._internal.TabMember._getLocked = true;
                        olExtension._internal.TabMember._internal.LockTabMembers();
                        var tabMembers_ = ol$Functions.grep(olData.TabInfoArray.TabMembers, function (member) {
                            try {
                                return ((!windowHandle || member.WindowHandle == windowHandle) && (!url || url == member.PageInfo.Url) && (!tabId || member.TabId == tabId) && (!applicationsSessionId || (olFunctions.IsFilledArray(member.PageInfo.Response.BeforeNavigateResponses) && member.PageInfo.Response.BeforeNavigateResponses[0].ApplicationsSessionId == applicationsSessionId)));
                            } catch (e) {
                                var alertErrorContent_ = new olFunctions.AlertContent('Filter GREP failed', e.message + '\n' + e.stack);
                                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                            }
                        });
                        olExtension._internal.TabMember._internal.FreeTabMembers();
                        olExtension._internal.TabMember._getLocked = false;
                        return tabMembers_;
                    } else {
                        setTimeout(function () {
                            olExtension._internal.TabMember.Filter(windowHandle, tabId, url, applicationsSessionId);
                        }, olOptions.General.Extension.TabInfoArraySleepInterval());
                    }
                } catch (e) {
                    var alertErrorContent_ = new olFunctions.AlertContent('Filter', e.message + '\n' + e.stack);
                    olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                }
            },
            Remove: function (tabMember, deleteCookies) {
                var alertContent_ = new olFunctions.AlertContent('Remove (tabMember)', 'Extension function');
                olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                if (typeof (deleteCookies) === 'undefined') {
                    deleteCookies = true;
                }
                try {
                    if (!olExtension._internal.TabMember._getLocked) {
                        olExtension._internal.TabMember._getLocked = true;
                        olExtension._internal.TabMember._internal.LockTabMembers();

                        //Delete cached script injections object as well
                        olWrap.api.tabsFramesScripts[tabMember.TabId] = {};

                        if (tabMember._internal.ExtensionFlags.ServiceResponded) {
                            olData.TabInfoArray.TabMembers = ol$Functions.grep(olData.TabInfoArray.TabMembers, function (member) {
                                try {
                                    return ((member.ApplicationsSessionId || !member._internal.ExtensionFlags.ServiceResponded) && (member.PageInfo.Url !== null) && (member.Id != tabMember.Id));
                                } catch (e) {
                                    return false;
                                }
                            });

                            olData._DebugStatisticsTabInfoArray = ol$Functions.grep(olData._DebugStatisticsTabInfoArray, function (member) {
                                return (member != tabMember.TabId);
                            });

                            if (olData._DebugStatisticsTabInfoArray.length == olData.TabInfoArray.TabMembers.length) {
                                olExtension.PrintStatistics_ = true;
                            }
                            else {
                                olExtension.PrintStatistics_ = false;
                            }
                        }
                        olExtension._internal.TabMember._internal.FreeTabMembers();
                        olExtension._internal.TabMember._getLocked = false;

                        if (deleteCookies && tabMember.PageInfo.Response.BeforeNavigateResponses && tabMember.PageInfo.Response.BeforeNavigateResponses[0].AppIdSpecified && !tabMember.PageInfo.Response.BeforeNavigateResponses[0].LogoutScriptExists) {
                            olExtension.Functions.RemoveCookies(tabMember);
                        }

                        olWrap.storage.asyncSetOlData(olData);
                    } else {
                        setTimeout(function () {
                            olExtension._internal.TabMember.Remove(tabMember, deleteCookies);
                        }, olOptions.General.Extension.TabInfoArraySleepInterval());
                    }
                } catch (e) {
                    var alertErrorContent_ = new olFunctions.AlertContent('Remove', e.message + '\n' + e.stack);
                    olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                    olExtension._internal.TabMember._internal.FreeTabMembers();
                }
            },
            RemoveOnErrorState: function (tabMember) {
                var alertContent_ = new olFunctions.AlertContent('RemoveOnErrorState (tabMember)', 'Extension function');
                olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                try {
                    if (tabMember._internal.ExtensionFlags.ErrorState && tabMember._internal.ExtensionFlags.DocumentReady) {
                        // FIX **************
                        // for https://henchman.atlassian.net/browse/ON-2601
                        // There was no sending message to page before this fix
                        // when ever there is RemoveOnErrorState function called
                        // now we are sending msg to page to show error prompt
                        var messageContent_ = new olExtension.DataConstructors.MessageContent(tabMember);
                        olExtension.Messages.Error(messageContent_);
                        // END FIX **********
                        olExtension._internal.TabMember.Remove(tabMember);
                    }
                } catch (e) {
                    var alertErrorContent_ = new olFunctions.AlertContent('RemoveOnErrorState', e.message + '\n' + e.stack);
                    olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                    olExtension._internal.TabMember._internal.FreeTabMembers();
                }
            },
            _getLogoutTMLocked: false,
            FilterLogoutInfoArray: function (windowHandle, tabId, applicationsSessionId) {
                try {
                    // var alertContent_ = new olFunctions.AlertContent('FilterLogoutInfoArray (tabMember)', 'Extension function');
                    // olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                    if (!olExtension._internal.TabMember._getLogoutTMLocked) {
                        olExtension._internal.TabMember._getLogoutTMLocked = true;
                        olExtension._internal.TabMember._internal.LockLogoutTabMembers();
                        var tabMembers_ = ol$Functions.grep(olData.LogoutInfoArray.TabMembers, function (member) {
                            try {
                                return ((!windowHandle || member.WindowHandle == windowHandle) && (!tabId || member.TabId == tabId) && (!applicationsSessionId || member.ApplicationsSessionId == applicationsSessionId));
                            } catch (e) {
                                return false;
                            }
                        });
                        olExtension._internal.TabMember._internal.FreeLogoutTabMembers();

                        olExtension._internal.TabMember._getLogoutTMLocked = false;
                        return tabMembers_;
                    } else {
                        setTimeout(function () {
                            olExtension._internal.TabMember.FilterLogoutInfoArray(windowHandle, tabId, applicationsSessionId);
                        }, olOptions.General.Extension.TabInfoArraySleepInterval());
                    }
                } catch (e) {
                    var alertErrorContent_ = new olFunctions.AlertContent('FilterLogoutInfoArray', e.message + '\n' + e.stack);
                    olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                }
            },
            GetLogoutTM: function (logoutWindowAttached, windowHandle, tabId) {
                try {
                    // var alertContent_ = new olFunctions.AlertContent('GetLogoutTM (tabMember)', 'Extension function');
                    // olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                    if (!olExtension._internal.TabMember._getLogoutTMLocked) {
                        olExtension._internal.TabMember._getLogoutTMLocked = true;
                        var returnValue_;
                        var tabMembers_ = new Array();

                        if (logoutWindowAttached) {
                            tabMembers_ = ol$Functions.grep(olData.LogoutInfoArray.TabMembers, function (member) {
                                try {
                                    return ((member._internal.ExtensionFlags.LogoutWindowAttached == logoutWindowAttached) && (!windowHandle || member.WindowHandle == windowHandle) && (!tabId || member.TabId == tabId));
                                } catch (e) {
                                    return false;
                                }
                            });
                        } else {
                            tabMembers_ = ol$Functions.grep(olData.LogoutInfoArray.TabMembers, function (member) {
                                return (member._internal.ExtensionFlags.LogoutWindowAttached == logoutWindowAttached);
                            });
                        }

                        if (olFunctions.IsFilledArray(tabMembers_)) {
                            returnValue_ = tabMembers_[0];
                            if (!returnValue_._internal.ExtensionFlags.LogoutWindowAttached) {
                                returnValue_._internal.ExtensionFlags.LogoutWindowAttached = true;
                                returnValue_._internal.ExtensionFlags.LogoutWindowAttachTime = new Date().getTime();
                            }
                            returnValue_.TabId = tabId;
                            returnValue_.WindowHandle = windowHandle;
                        } else {
                            returnValue_ = null;
                        }
                        olExtension._internal.TabMember._getLogoutTMLocked = false;

                        return returnValue_;
                    } else {
                        setTimeout(function () {
                            olExtension._internal.TabMember.GetLogoutTM(logoutWindowAttached, windowHandle, tabId);
                        }, olOptions.General.Extension.TabInfoArraySleepInterval());
                    }
                } catch (e) {
                    var alertErrorContent_ = new olFunctions.AlertContent('GetLogoutTM', e.message + '\n' + e.stack);
                    olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                }
            },
            RemoveLogout: function (tabMember) {
                try {
                    // var alertContent_ = new olFunctions.AlertContent('RemoveLogout (tabMember)', 'Extension function');
                    // olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                    if (!olExtension._internal.TabMember._getLogoutTMLocked) {
                        olExtension._internal.TabMember._getLogoutTMLocked = true;
                        olExtension._internal.TabMember._internal.LockLogoutTabMembers();
                        // olExtension.Functions.HandleCookiesArray();
                        olExtension.Functions.RemoveCookies(tabMember);
                        olData.LogoutInfoArray.TabMembers = ol$Functions.grep(olData.LogoutInfoArray.TabMembers, function (member) {
                            try {
                                return ((member.PageInfo.Url !== null) && (member.Id != tabMember.Id));
                            } catch (e) {
                                return false;
                            }
                        });
                        olExtension._internal.TabMember._getLogoutTMLocked = false;
                        olExtension._internal.TabMember._internal.FreeLogoutTabMembers();
                    } else {
                        setTimeout(function () {
                            olExtension._internal.TabMember.RemoveLogout(tabMember);
                        }, olOptions.General.Extension.TabInfoArraySleepInterval());
                    }
                } catch (e) {
                    var alertErrorContent_ = new olFunctions.AlertContent('Remove', e.message + '\n' + e.stack);
                    olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                }
            },
            RemoveFromCookieArray: function (tabMember) {
                try {
                    // var alertContent_ = new olFunctions.AlertContent('RemoveFromCookieArray (tabMember)', 'Extension function');
                    // olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                    if (!olData.CookiesInfoArray.TabMembersLocked) {
                        olData.CookiesInfoArray.TabMembersLocked = true;
                        olData.CookiesInfoArray.TabMembers = ol$Functions.grep(olData.CookiesInfoArray.TabMembers, function (member) {
                            try {
                                return ((member.PageInfo.Url !== null) && (member.Id != tabMember.Id));
                            } catch (e) {
                                return false;
                            }
                        });
                        olData.CookiesInfoArray.TabMembersLocked = false;

                    } else {
                        setTimeout(function () {
                            olExtension._internal.TabMember.RemoveFromCookieArray(tabMember);
                        }, olOptions.General.Extension.TabInfoArraySleepInterval());
                    }
                } catch (e) {
                    olData.CookiesInfoArray.TabMembersLocked = false;
                    var alertErrorContent_ = new olFunctions.AlertContent('RemoveFromCookieArray', e.message + '\n' + e.stack);
                    olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                }
            },
            GetElementVariableReplacement: function (definedNamesArray, variables) {
                try {
                    // var alertContent_ = new olFunctions.AlertContent('GetElementVariableReplacement', 'Extension function');
                    // olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                    var elValue_ = null;
                    if (olFunctions.IsFilledArray(variables)) {
                        for (var i = 0; i < variables.length; i++) {
                            var currentVariable_ = variables[i];
                            if (ol$Functions.inArray(currentVariable_.DefinedName, definedNamesArray) > -1) {
                                elValue_ = currentVariable_.Value;
                                break;
                            }
                        }
                    }
                    return elValue_;
                } catch (e) {
                    var alertErrorContent_ = new olFunctions.AlertContent('GetElementVariableReplacement', e.message + '\n' + e.stack);
                    olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                }
            },
            HandleResourceEnd: function (tabMember) {
                try {
                    // var alertContent_ = new olFunctions.AlertContent('HandleResourceEnd', 'Extension function');
                    // olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                    if (tabMember.ApplicationsSessionId) {
                        var tabMembers_ = olExtension._internal.TabMember.Filter(tabMember.WindowHandle, null, null, tabMember.ApplicationsSessionId);
                        if (tabMembers_.length == 1) {
                            // instead of quitting we now check if there is logout script
                            var requestContent_;
                            if (tabMember.PageInfo.Response.BeforeNavigateResponses[0].LogoutScriptExists) {
                                // if there is a logout script we need tab info stored in separate array for logouts
                                // also we need closing responses
                                if (tabMember.PageInfo.ClosingResponse) {
                                    // we have closing responses from window closing request                           
                                    olData.LogoutInfoArray.TabMembers.push(tabMember);
                                    olExtension._internal.TabMember.Remove(tabMember, false);

                                    requestContent_ = new olExtension.DataConstructors.RequestContent(tabMember);
                                    olExtension.Service.NewWindowRequestSend(requestContent_);
                                } else {
                                    requestContent_ = new olExtension.DataConstructors.RequestContent(tabMember);
                                    olExtension.Service.QuittingResourceSessionRequestSend(requestContent_);
                                }
                            } else {
                                requestContent_ = new olExtension.DataConstructors.RequestContent(tabMember);
                                olExtension.Service.QuittingResourceSessionRequestSend(requestContent_);
                            }
                        } else {
                            olExtension._internal.TabMember.Remove(tabMember);
                            if (tabMember.CloseTab) {
                                tabMember.TargetTab.close();
                            }
                        }
                    } else {
                        olExtension._internal.TabMember.Remove(tabMember, false);
                    }
                } catch (e) {
                    olExtension._internal.TabMember.Remove(tabMember);
                    var alertErrorContent_ = new olFunctions.AlertContent('HandleResourceEnd', e.message + '\n' + e.stack);
                    olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                }
            }
        },
        _escapeRegExp: function (string) {
            return string.replace(/([.*+?^=!:${}()|\[\]\/\\])/g, "\\$1");
        },
        _replaceAll: function (string, find, replace) {
            return string.replace(new RegExp(olExtension._internal._escapeRegExp(find), 'g'), replace);
        },
        StoreOneLogFormsFetch: async function () {
            try {
                var alertContent_ = new olFunctions.AlertContent('StoreOneLogFormsFetch', 'Extension function');
                olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                var ExtensionInfo_ = olWrap.getExtensionInfo();

                var resourcesUrlPrefix_ = '';
                const response = await fetch(resourcesUrlPrefix_ + 'resource.html?version=' + ExtensionInfo_.version)
                if (response.status === 200) {
                    var response_ = await response.text();
                    response_ = olExtension._internal.Translate(response_);

                    var bodyMisc_ = response_.match(/<olEnvelopeMisc[^>]*>[\s\S]*<\/olEnvelopeMisc>/gi)[0].replace(/<olEnvelopeMisc[^>]*>/gi, '').replace(/<\/olEnvelopeMisc>/gi, '');
                    olWrap.storage.asyncSetItem('olMisc', bodyMisc_);

                    var body_ = response_.match(/<olEnvelopeResources[^>]*>[\s\S]*<\/olEnvelopeResources>/gi)[0].replace(/<olEnvelopeResources[^>]*>/gi, '').replace(/<\/olEnvelopeResources>/gi, '');
                    olWrap.storage.asyncSetItem('olHtml', body_);

                    var bodyToolbar_ = response_.match(/<olEnvelopeToolbar[^>]*>[\s\S]*<\/olEnvelopeToolbar>/gi)[0].replace(/<olEnvelopeToolbar[^>]*>/gi, '').replace(/<\/olEnvelopeToolbar>/gi, '');
                    olWrap.storage.asyncSetItem('olToolbarDiv', bodyToolbar_);

                    var bodyTimeout_ = response_.match(/<olEnvelopeResourceTimeout[^>]*>[\s\S]*<\/olEnvelopeResourceTimeout>/gi)[0].replace(/<olEnvelopeResourceTimeout[^>]*>/gi, '').replace(/<\/olEnvelopeResourceTimeout>/gi, '');
                    olWrap.storage.asyncSetItem('olTimeout', bodyTimeout_);

                    var bodyError_ = response_.match(/<olEnvelopeError[^>]*>[\s\S]*<\/olEnvelopeError>/gi)[0].replace(/<olEnvelopeError[^>]*>/gi, '').replace(/<\/olEnvelopeError>/gi, '');
                    olWrap.storage.asyncSetItem('olError', bodyError_);

                    var bodyStorePassword_ = response_.match(/<olEnvelopeStorePassword[^>]*>[\s\S]*<\/olEnvelopeStorePassword>/gi)[0].replace(/<olEnvelopeStorePassword[^>]*>/gi, '').replace(/<\/olEnvelopeStorePassword>/gi, '');
                    olWrap.storage.asyncSetItem('olStorePassword', bodyStorePassword_);

                    var bodyWebControlMessage_ = response_.match(/<olEnvelopeWebControlMessage[^>]*>[\s\S]*<\/olEnvelopeWebControlMessage>/gi)[0].replace(/<olEnvelopeWebControlMessage[^>]*>/gi, '').replace(/<\/olEnvelopeWebControlMessage>/gi, '');
                    olWrap.storage.asyncSetItem('olWebControlMessage', bodyWebControlMessage_);

                    var bodyChooseApplication_ = response_.match(/<olEnvelopeChooseApplication[^>]*>[\s\S]*<\/olEnvelopeChooseApplication>/gi)[0].replace(/<olEnvelopeChooseApplication[^>]*>/gi, '').replace(/<\/olEnvelopeChooseApplication>/gi, '');
                    olWrap.storage.asyncSetItem('olChooseApplication', bodyChooseApplication_);
                }
                else {
                    var alertErrorContent_ = new olFunctions.AlertContent('StoreOneLogFormsFetch resource request failed:' + resourceXmlBody_.statusText, e.message + '\n' + e.stack);
                }
            } catch (e) {
                var alertErrorContent_ = new olFunctions.AlertContent('StoreOneLogFormsFetch', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
            }
        },

        Username: null,
        UsernameResolved: false,
        Translate: function (text) {
            if (olData.LanguageItems && olFunctions.IsFilledArray(olData.LanguageItems)) {
                for (var i = 0; i < olData.LanguageItems.length; i++) {
                    var languageItem_ = olData.LanguageItems[i];
                    text = text.replace(new RegExp('\\b' + String(languageItem_.Key) + '\\b', 'g'), String(languageItem_.Value));
                }
            }
            return text;
        },
        _logoffLocked: false,
        windowObjectReference: null,
        tabObjectReference: null,
        OpenLogoutWindow: function (tabId) {
            try {
                olWrap.windows.create(olExtension.Data.LogoutUrl, function (win) {
                    var elapsedTime_;
                    if (olOptions.DebugApiMess()) {
                        elapsedTime_ = 33000;
                    } else {
                        elapsedTime_ = 10050;
                    }
                    setTimeout(function () {
                        var alertErrorContent_;
                        try {
                            chrome.windows.remove(win.id);
                        } catch (e) {
                            alertErrorContent_ = new olFunctions.AlertContent('Failed to close window', e.message + '\n' + e.stack);
                            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                        }
                    }, elapsedTime_)
                });
            } catch (e) {
                var alertErrorContent_ = new olFunctions.AlertContent('OpenLogoutWindow', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
            }
        },
        HandleLogoutWindow: function (event) {
            try {
                var alertContent_ = new olFunctions.AlertContent('Handle logout window', 'Extension function');
                olFunctions.Alert(olOptions.DebugFunctions(), alertContent_);
                var targetId = event.target.getId();

                if (olData.LogoutTabs.indexOf(targetId) == -1) {
                    olData.LogoutTabs.push(targetId);

                    olWrap.storage.asyncSetOlData(olData)
                    //Once logout window is opened we take Logout tab member and handle logout
                    var logoutTabMember_ = olExtension._internal.TabMember.GetLogoutTM(false, olExtension.Data.WindowHandle, event.target.getId());
                    if (logoutTabMember_) {
                        setTimeout(function () {
                            try {
                                if (logoutTabMember_.CloseTab) {
                                    // logoutTabMember_.TargetTab.close();
                                    var parentTab = logoutTabMember_.TargetTab;
                                    parentTab.close();
                                }

                                logoutTabMember_.TargetTab = event.target;
                                var lastUrl_ = logoutTabMember_._internal.GetLastUrl();

                                // if ClosingResponse.LastURL is not defined set last url to last url from TabMember
                                if (logoutTabMember_.PageInfo.ClosingResponse && !logoutTabMember_.PageInfo.ClosingResponse.LastURL) {
                                    logoutTabMember_.PageInfo.ClosingResponse.LastURL = lastUrl_;
                                }
                                logoutTabMember_.TargetTab.navigate(lastUrl_);
                            } catch (error) {

                            }
                        })
                    } else {
                        event.target.close();
                    }
                }
            } catch (e) {
                var alertErrorContent_ = new olFunctions.AlertContent('HandleLogoutWindow', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
            }
        }
    },
    Cleaner: {
        _CloseCoverWindow: false,
        _ResetTime: null,
        _LogoutMemberTabIds: [],
        Reset: function () {
            olExtension.Cleaner._ResetTime = new Date().getTime() + 3000;
        },
        Start: function () {
            setInterval(function () {
                olExtension.Cleaner._Clean();
            }, 3000);
        },
        _Clean: function () {
            if (olExtension.Cleaner._CloseCoverWindow) {
                // if needed close cover window and wait next iteration
                olExtension.Cleaner._CloseCoverWindow = false;
                if (olExtension._internal.tabObjectReference) {
                    olExtension._internal.tabObjectReference.close();
                }
            } else {
                var currentTime_ = new Date().getTime();
                if (currentTime_ > olExtension.Cleaner._ResetTime) {
                    olExtension.Cleaner.Reset();
                    // Clean now
                    olExtension.Cleaner._CloseCoverWindow = true;

                    var getTabsCallback = function (tabs) {
                        try {
                            for (var j = 0; j < tabs.length; j++) {
                                var tabId_ = tabs[j].getId();
                                // var tabId_ = tabs[j].id;

                                var logoutListMembers_ = ol$Functions.grep(olExtension.Cleaner._LogoutMemberTabIds, function (member) {
                                    return (member.Id == tabId_);
                                });
                                if (olFunctions.IsFilledArray(logoutListMembers_)) {
                                    var logoutTabMember_ = logoutListMembers_[0];
                                    // clean persistent attached window
                                    var elapsedTime_ = null;
                                    if (olOptions.DebugApiMess()) {
                                        elapsedTime_ = 33000;
                                    } else {
                                        elapsedTime_ = 10050;
                                    }
                                    var currentTime_ = new Date().getTime();
                                    //TODO ADDED: check if tabs[j] exists
                                    if (currentTime_ > (logoutTabMember_.AttachTime + elapsedTime_)) {
                                        //TODO Check this!!!
                                        try {
                                            tabs[j].close();
                                        } catch (e) {
                                        }
                                    } else {
                                        olExtension.Cleaner._CloseCoverWindow = false;
                                    }
                                }
                            }
                        } catch (e) {
                            var alertErrorContent_ = new olFunctions.AlertContent('_Clean', e.message + '\n' + e.stack);
                            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                        }
                    };

                    olWrap.windows.getAll(function (wins) {
                        for (var i = 0; i < wins.length; i++) {
                            wins[i].getTabs(getTabsCallback);
                            // getTabsCallback(wins[i].tabs);
                        }
                    });
                }
            }
        }
    },
    BasicAuthentication: {
        _internal: {
            RefreshTabIdArray: [],
            AddToRefreshTabIdArray: function (tabId) {
                try {
                    olExtension.BasicAuthentication._internal.RefreshTabIdArray.push(tabId);
                } catch (e) {
                    var alertErrorContent_ = new olFunctions.AlertContent('AddToRefreshTabIdArray', e.message + '\n' + e.stack);
                    olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                }
            },
            CheckRefreshTabId: function (tabId) {
                var length_ = olExtension.BasicAuthentication._internal.RefreshTabIdArray.length;
                olExtension.BasicAuthentication._internal.RefreshTabIdArray = ol$Functions.grep(olExtension.BasicAuthentication._internal.RefreshTabIdArray, function (member) {
                    return member != tabId;
                });
                if (length_ == olExtension.BasicAuthentication._internal.RefreshTabIdArray.length) {
                    return false;
                }
                return true;
            },
            RemoveFromRefreshTabIdArray: function (tabId) {
                try {
                    olExtension.BasicAuthentication._internal.RefreshTabIdArray = ol$Functions.grep(olExtension.BasicAuthentication._internal.RefreshTabIdArray, function (member) {
                        return member != tabId;
                    });
                } catch (e) {
                    var alertErrorContent_ = new olFunctions.AlertContent('RemoveFromRefreshTabIdArray', e.message + '\n' + e.stack);
                    olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
                }
            }
        },
        CheckUrlLoggedInBasicAuth: async function (tabMember, url, username, password) {
            //double basic authentication check
            /*
              try {
                     fetch(url, {
                      method: 'GET', 
                      cache: 'no-cache',                    
                      headers: {
                      'Content-Type': 'application/json'
                      },
                  }).then (response => {                
                              if(response.status!==401){
                                  headers = new Headers();
                                  headers.set('Authorization', 'Basic ' + btoa(username + ":" + password));
                                  headers.set("Content-Type", "application/json");
                                   fetch(url + '/', {
                                      method: 'GET',                     
                                      cache: 'no-cache',                            
                                      headers: headers
                                  }).then (response => {
                                          if(response.status!==401) {
                                          tabMember.TargetTab.navigate(url);
                                          } 
                                      })
                              } 
                          })                        
              } catch (e) {
                  var alertErrorContent_ = new olFunctions.AlertContent('Login', e.message + '\n' + e.stack);
                  olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
              }   
              */

        },
        HandleBasicAuthentication: function (tabMember, variables) {
            try {
                var bnr0_ = tabMember.PageInfo.Response.BeforeNavigateResponses[0];
                var username_ = olExtension._internal.TabMember.GetElementVariableReplacement(tabMember.PageInfo.DefinedNames.UsernameArray, variables);
                var password_ = olExtension._internal.TabMember.GetElementVariableReplacement(tabMember.PageInfo.DefinedNames.PasswordArray, variables);

                //Basic authentication form exists?
                var currentForm_;
                if (bnr0_.Forms && olFunctions.IsFilledArray(bnr0_.Forms)) {
                    for (var i = 0; i < bnr0_.Forms.length; i++) {
                        currentForm_ = bnr0_.Forms[i];
                        if (currentForm_.Basic) {
                            tabMember._internal.ExtensionFlags.BasicAuthFormExist = true;
                            tabMember._internal.ExtensionFlags.BasicAuthURLs = currentForm_.URL;
                            break;
                        }
                    }
                }
                if (tabMember._internal.ExtensionFlags.BasicAuthFormExist) {
                    if (!tabMember._internal.ExtensionFlags.BasicAuthDoneOrNotNeeded && !tabMember._internal.ExtensionFlags.BasicAuthLoginSent && username_ && password_) {
                        tabMember._internal.ExtensionFlags.BasicAuthLoginSent = true;
                        if (olFunctions.IsFilledArray(tabMember._internal.ExtensionFlags.BasicAuthURLs)) {
                            tabMember._internal.ExtensionFlags.NumberOfBasicAuthUrls = tabMember._internal.ExtensionFlags.BasicAuthURLs.length;
                            for (i = 0; i < tabMember._internal.ExtensionFlags.BasicAuthURLs.length; i++) {
                                var formUrl_ = tabMember._internal.ExtensionFlags.BasicAuthURLs[i];
                                olExtension.BasicAuthentication.Login(tabMember, username_, password_, formUrl_);
                            }
                        }
                    } else {
                        olExtension.BasicAuthentication._internal.RemoveFromRefreshTabIdArray(tabMember.TabId);
                        olExtension.BasicAuthentication.CheckUrlLoggedInBasicAuth(tabMember, tabMember.PageInfo.Url, username_, password_);
                    }
                } else {
                    tabMember._internal.ExtensionFlags.BasicAuthDoneOrNotNeeded = true;
                }
            } catch (e) {
                var alertErrorContent_ = new olFunctions.AlertContent('HandleBasicAuthentication', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
            }
        },
        Login: function (tabMember, username, password, url) {
            try {

                headers = new Headers();


            } catch (e) {
                var alertErrorContent_ = new olFunctions.AlertContent('Login', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
            }
        }
    },
    Data: {
        JsonMv3TestingScript: null,
        SW_Initialised: false,
        OneLogInfoPannelUrl: null,
        LogoutInfoWindow: null,
        WindowHandle: null,

        // ResourcesUrlPrefix: '',
        // LogoutUrl: 'https://devtest.onelog.com/mobview/olResources/logout.html',
        // LogoutInfoUrl: 'https://devtest.onelog.com/mobview/olResources/logoutInfo.html',
        // RuntimeDataUrl: 'https://devtest.onelog.com/mobview/olResources/olRuntimeData.json',

        ResourcesUrlPrefix: 'https://cloud.onelog.com/extensions/olResources/',
        LogoutUrl: 'https://cloud.onelog.com/extensions/olResources/logout.html',
        LogoutInfoUrl: 'https://cloud.onelog.com/extensions/olResources/logoutInfo.html',
        RuntimeDataUrl: 'https://cloud.onelog.com/extensions/olResources/olRuntimeData.json',

        NativeEdgePort: null,
        NativeChromePort: null
    },
    DataConstructors: {
        AjaxParams: function (data) {
            try {
                var self = this;
                self.ProcessData = false;
                self.Async = true;
                self.Type = 'POST';
                self.Url = olOptions.General.Extension.ServerUrl();
                self.ContentType = 'jsonp';
                self.Data = data;
                self.Timeout = olOptions.General.Extension.AjaxCallTimeout();
            } catch (e) {
                var alertErrorContent_ = new olFunctions.AlertContent('AjaxParams', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
            }
        },
        MessageContent: function (tabMember, responseContent) {
            try {
                var self = this;
                if (typeof (tabMember) === 'undefined')
                    tabMember = null;
                if (typeof (responseContent) === 'undefined')
                    responseContent = null;
                if (tabMember) {
                    self.TabMember = tabMember;
                    self.TabMessageContent = {
                        ApplicationsSessionId: tabMember.ApplicationsSessionId,
                        ApplicationName: tabMember.ApplicationName,
                        PageFlags: tabMember._internal.PageFlags,
                        PageInfo: tabMember.PageInfo,
                        ResponseContent: responseContent,
                        SelectedUser: tabMember._internal.SelectedUser ? tabMember._internal.SelectedUser : {},
                    };
                }
            } catch (e) {
                var alertErrorContent_ = new olFunctions.AlertContent('MessageContent', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
            }
        },
        PageInfo: function (url) {
            try {
                var self = this;
                self.Url = url;
                self.LastUrl = url;
                self.TestingJsonScripts = false;
                self.Response = {
                    BeforeNavigateResponses: null,
                    CheckMatterResponse: null,
                    DocumentCompleteResponse: null,
                    ErrorState: null,
                    GetAppIdListResponse: null,
                    GetLocalPersonalCodeListResponse: null,
                    LanguageItems: null,
                    NavigateCompleteResponses: null,
                    PulseResponse: null,
                    QuittingResponse: null,
                    SetCommentEditResponse: null,
                    SetCommonDetailsResponse: null,
                    SetPersonalDetailsResponse: null,
                    TestResourceSelectResponse: null,
                    ValidateMatterResponse: null,
                    WindowClosingResponses: null
                };
                self.ClosingResponse = null;
                self.DefinedNames = {
                    UsernameArray: [],
                    PasswordArray: [],
                    MatterArray: [],
                    CustomArray: [],
                    TimeKeeperArray: [],
                    CommentArray: []
                };
            } catch (e) {
                var alertErrorContent_ = new olFunctions.AlertContent('PageInfo', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
            }
        },
        RequestContent: function (tabMember, parameters) {
            try {
                var self = this;
                if (typeof (tabMember) === 'undefined')
                    tabMember = null;
                if (typeof (parameters) === 'undefined')
                    parameters = null;

                self.TabMember = tabMember;
                self.Parameters = parameters;
            } catch (e) {
                var alertErrorContent_ = new olFunctions.AlertContent('RequestContent', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
            }
        },
        SendRequestParameters: function (requestName, ajaxData, requestContent, requestSuccessFunction) {
            try {
                var self = this;
                self.RequestName = requestName;
                self.AjaxParam = new olExtension.DataConstructors.AjaxParams(ajaxData);
                self.RequestContent = requestContent;
                self.RequestSuccessFunction = requestSuccessFunction;
            } catch (e) {
                var alertErrorContent_ = new olFunctions.AlertContent('SendRequestParameters', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
            }
        },
        ExtensionFlags: function () {
            var self = this;
            self.BeforeNavigateRequestSent = false;
            self.BasicAuthDoneOrNotNeeded = false;
            self.BasicAuthLoginSent = false;
            self.BasicAuthFormExist = false;
            self.BasicAuthURLs = [];
            self.ClosingResponsesSent = false;
            self.DocumentReady = false;
            self.LogoutWindowAttached = false;
            self.LogoutWindowAttachTime = null;
            self.MatterDetailFilled = false;
            self.NumberOfBasicAuthUrls = 0;
            self.PersonalCodeFilled = false;
            self.ServiceResponded = false;
            self.TabRemoved = false;
            self.Error = false;
            self.ErrorState = false;
            self.DocumentCompleteResponseReceived = false;
            self.DocumentCompleteRequestSent = false;
            self.NavigateCompleteRequestSent = false;
            self.FormFillRequestNeeded = false;
            self.FormFillRequestSent = false;
            self.ProcessSubframes = false;
            self.ProcessSubframesDomains = [];
        },
        PageFlags: function (TabId) {
            var self = this;
            self.ExtensionEnvironment = true;
            self.MobileEnvironment = '';
            self.BrowserName = olData.BrowserName;
            var extensionVersion_ = olWrap.getExtensionInfo().version;
            self.ExtensionVersion = extensionVersion_;
            //Version for scripts to use if there is need to separate scripts by extension versions
            self.ExtensionVersionComparable = extensionVersion_.substring(0, 10) + (extensionVersion_.substring(10).length == 2 ? '0' + extensionVersion_.substring(10) : extensionVersion_.substring(10));

            self.Comment = '';
            self.CommentChanged = false;
            self.CommentRequest = false;
            self.InOfflineMode = false;
            self.MatterNeeded = false;
            self.MatterDone = false;
            self.MatterSkipped = false;
            self.MatterChanged = false;
            self.MatterNumber = '';
            self.MatterFormat = '';
            self.FinalMatterValue = '';
            self.MatterFreeInput = false;
            self.UseMatterInScript = false;
            self.PersonalCodeNeeded = false;
            self.PersonalCodeDone = false;
            self.TimekeeperNumber = '';
            self.TimekeeperNumberForSite = '';
            self.TimekeeperChanged = false;
            self.LogonNeeded = false;
            self.LogonNeededForUsername = false;
            self.LogonNeededForPassword = false;
            self.LogonCommonNeeded = false;
            self.LogonCommonNeededForUsername = false;
            self.LogonCommonNeededForPassword = false;
            self.LogonDetailsFreeInput = false;
            self.LogonDone = false;
            self.LogonSkipped = false;
            //Changed to null so preference on option page can override this on first setting of the toolbar
            self.ToolbarMinimized = null;
            self.Unauthorised = false;
            self.LicenceLimitReachedNeeded = false;
            self.LicenceLimitReachedDone = false;
            self.PooledDetailsLimitReachedNeeded = false;
            self.PooledDetailsLimitReachedDone = false;
            self.TabId = TabId;
            self.PmtUser = true;
            self.PmtAdmin = false;
            self.PmtAutomatic = false;
            self.PassDetailsToSite = true;
            self.LoginType = 'User';
            self.LogonUsingSharedDetailsPossible = false;
            self.LogonSharingDetailsPossible = false;
            self.IsNewPassword = false;
            self.FormfillRequestSent = false;
            self.ProcessSubframes = false;
        },
        TabMember: function (tabId, url, framesArray) {
            try {
                if (typeof (framesArray) === 'undefined') {
                    framesArray = [];
                }

                var self = this;
                self.LastTimeProcessed = 0;
                self.Id = olWrap.Functions.Guid();
                self.ApplicationsSessionId = null;
                self.ApplicationName = null;
                self.BrowserType = olData.BrowserName;
                self.CloseTab = false;
                self.DocumentTitle = null;
                self.PageInfo = new olExtension.DataConstructors.PageInfo(url);
                self.WindowHandle = olExtension.Data.WindowHandle;
                self.WindowId = tabId + '-' + olExtension.Data.WindowHandle;
                self.TabId = tabId;
                self.Domains = [];
                self.InjectionParameters = {};
                self.SubframesOnPageToProcess = framesArray;

                // TargetTab gets filled on document complete event also, since this way isn't working for Chrome
                olWrap.tabs.getAll(function (tabs) {
                    for (var i = 0; i < tabs.length; i++) {
                        if (tabs[i].getId() == tabId) {
                            self.TargetTab = tabs[i];
                        }
                    }
                });

                var extensionFlags_ = new olExtension.DataConstructors.ExtensionFlags();
                var pageFlags_ = new olExtension.DataConstructors.PageFlags(tabId);

                self._internal = {
                    ExtensionFlags: extensionFlags_,
                    PageFlags: pageFlags_,
                    GetUrl: function () {
                        try {
                            return self.PageInfo.Url;
                        } catch (error) {

                        }
                    },
                    GetLastUrl: function () {
                        try {
                            return self.PageInfo.LastUrl;
                        } catch (error) {

                        }
                    }
                };
            } catch (e) {
                var alertErrorContent_ = new olFunctions.AlertContent('TabMember', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
            }
        }
    },
    PrintStatistics_: true,
    _initialize: function () {
        try {
            //data initialization
            if (olData.Initialized) {
            } else {
                olData.Initialize();
            }

            if (olOptions.DebugStatistics()) {
                var timeInMs_ = olOptions.DebugStatisticsPeriod() * 60000;
                setTimeout(function () {
                    //Get data from tabs           
                    for (var i = 0; i < olData.TabInfoArray.TabMembers.length; i++) {
                        var currentTabMember_ = olData.TabInfoArray.TabMembers[i];
                        var messageContent_ = new olExtension.DataConstructors.MessageContent(currentTabMember_);
                        olExtension.Messages._GetStatisticsFromPage(messageContent_);
                    }
                    var printStats = function () {
                        var printStats_ = setInterval(
                            function () {
                                if (olOptions.DebugStatistics()) {
                                    clearInterval(printStats_);
                                    olWrap.Console.Statistics(olData._DebugStatistics);
                                    olExtension.Service.ExtensionStatisticsRequestSend();
                                }
                            }, 1000
                        )
                        olData._DebugStatistics = {
                            Transactions: [],
                            Summary: []
                        };
                    }
                    setTimeout(printStats, 0);
                }, timeInMs_);
            }

            if (olOptions.DebugApiMess()) {
                var ExtensionInfo_ = olWrap.getExtensionInfo();
                olWrap.Console.Log(olData.BrowserName + ' version:' + ExtensionInfo_.version);
            };

            switch (olWrap.browser.getName()) {
                case olWrap.browser.name.firefox:
                    callback = function () {
                        var allTabsIds_ = [];
                        olData.CreatedTabs = [];
                        olWrap.tabs.getAll(function (tabs) {
                            var olDataTabsIds_ = JSON.parse(JSON.stringify(olData.TabInfoArray.TabMembers));
                            for (var i = 0; i < tabs.length; i++) {
                                var filterMembers_ = olExtension._internal.TabMember.Filter(null, tabs[i].id, null, null);
                                if (olFunctions.IsFilledArray(filterMembers_)) {
                                    filterMembers_[0].TargetTab = tabs[i]
                                }
                                olData.CreatedTabs.push(tabs[i].getId());
                                allTabsIds_.push(tabs[i].getId())
                            }

                            for (var k = 0; k < olDataTabsIds_.length; k++) {
                                var currentTabmember_ = olDataTabsIds_[k];
                                if (ol$Functions.inArray(currentTabmember_.TabId, allTabsIds_) == -1) {

                                    var tabMembers_ = olExtension._internal.TabMember.Filter(null, currentTabmember_.TabId, null, null);
                                    for (var q = 0; q < tabMembers_.length; q++) {

                                        olData.TabInfoArray.TabMembers = ol$Functions.grep(olData.TabInfoArray.TabMembers, function (member) {
                                            try {
                                                return (member.Id != tabMembers_[q].Id);
                                            } catch (e) {
                                            }
                                        });
                                    }
                                } else {
                                }
                            }

                            olExtension.Data.SW_Initialised = true;
                            olWrap.storage.asyncSetOlData(olData);
                        })
                    }

                    olWrap.storage.asyncGetOlData(callback);
                    break;
                default:
                    break;
            }
            olExtension.Data.WindowHandle = olWrap.browser.getName();

            //Options data from organisation
            setTimeout(function () {
                olExtension.Service.ExtensionOptionsRequestSend();
            }, 1000);

            //Language items
            setTimeout(function () {
                olExtension.Service.LanguageItemsRequestSend();
            }, 1000);
        } catch (e) {
            var alertErrorContent_ = new olOptions.DataConstructors.AlertContent('_initialize ', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    Initialize: function () {
        olOptions.InitializeOptionsData(function () {
            try {
                //Native Messaging Port
                switch (olWrap.browser.getName()) {
                    case olWrap.browser.name.chrome:
                        switch (olExtension.Version) {
                            case 'TerminalServer':
                                olExtension.Data.NativeChromePort = chrome.runtime.connectNative("com.its.onelog");
                                break;
                            default:
                                break;
                        }
                        break;
                    case olWrap.browser.name.edg:
                        switch (olExtension.Version) {
                            case 'TerminalServer':
                                olExtension.Data.NativeChromePort = chrome.runtime.connectNative("com.its.onelog");
                                break;
                            default:
                                break;
                        }
                        break;
                }

                switch (olExtension.Version) {
                    case 'TerminalServer':
                        setTimeout(function () {
                            olExtension.Service.UsernameRequestSend();
                        }, 1000);
                        break;
                    default:
                        olExtension._initialize();
                        break;
                }
            } catch (e) {
                var alertErrorContent_ = new olFunctions.AlertContent('Initialize', e.message + '\n' + e.stack);
                olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
            }
        });
    }
};
/**
 *  one time extension initialization/
 */

olExtension.Version = 'Default';
olExtension.Initialize();