/**
 *  OneLog Functions
 */

var olFunctions = {
    Mobile: {
        decodeUTF8: function (utftext) {
            try {
                var returnUTFString_ = "";
                for (var i = 0; i < utftext.length; i++) {
                    returnUTFString_ += String.fromCharCode(utftext[i]);
                }
            } catch (e) {
                returnUTFString_ = '';
            } finally {
                return returnUTFString_;
            }
        },
    },
    jsAtob: function (code) {
        try {
            switch (typeof code) {
                case 'string':
                    return atob(code);
                default:
                    try {
                        return String.fromCharCode.apply(null, new Uint8Array(code));
                    } catch (e) {
                        var alertContent_ = new olFunctions.AlertContent('Mobile', 'Could not convert object to code!');
                        olFunctions.Alert(olOptions.Errors(), alertContent_);
                        return '';
                    }
            }
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('jsAtob', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    Alert: function (bShow, alertContent, type) {
        try {
            var text_ = alertContent.Message ? alertContent.Title + ' - ' + alertContent.Message : alertContent.Title;
            if (bShow) {
                if (olOptions) {
                    if (olOptions.Debug.Alerts()) {
                        alert(text_);
                    }
                    var browserName_ = olWrap.browser.getName();
                    switch (browserName_) {
                        case 'ie':
                            olWrap.Console.Log(text_);
                            break;
                        default:
                            console.warn(text_);
                    }
                } else {
                    alert(text_);
                }
            }
        } catch (e) {
            olWrap.Console.Log(text_, type);
        }
    },
    AlertContent: function (title, message) {
        var self = this;
        if (typeof (message) === 'undefined')
            message = '';
        self.Title = title;
        self.Message = message;

        var date_ = new Date();
        var timestamp_ = Math.round(+date_ / 1000) + ':' + Math.round(+date_ % 1000);
        self.TimeStamp = timestamp_.slice(5);
    },
    IsFilledArray: function (ar) {
        try {
            if (ar.length && ar.length > 0) {
                return true;
            } else {
                return false;
            };
        } catch (e) {
            return false;
        }
    },
    olAddEventListener: function (element, event, eventFunction) {
        try {
            if (!element.data) {
                element.data = [];
            }
            var eventObjects_ = [];
            if (element.data) {
                eventObjects_ = ol$Functions.grep(element.data, function (member) {
                    return (member.event == event);
                });
            }

            if (element.addEventListener) {
                for (var i = 0; i < eventObjects_.length; i++) {
                    element.removeEventListener(event, eventObjects_[i].eventFunction);
                }
                element.addEventListener(event, eventFunction);


            } else if (elementF.attachEvent) {
                for (var i = 0; i < eventObjects_.length; i++) {
                    element.detachEvent(event, eventObjects_[i].eventFunction);
                }
                element.attachEvent('on' + event, eventFunction);
            }
            element.data = ol$Functions.grep(element.data, function (member) {
                return (member.event != event);
            });

            element.data.push({
                event: event,
                eventFunction: eventFunction
            })
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('olAddEventListener', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    GetDomainFromUrl: function (url) {
        try {
            if (url) {
                var matches_ = url.match(/:\/\/(.[^/:]+)/);
                var matches1_ = ('http://' + url).match(/:\/\/(.[^/:]+)/);


                return ((matches_ !== null && typeof matches_[1] != 'undefined') ? matches_[1] : matches1_[1]);
            }
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('GetDomainFromUrl', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    GetDomainFromUrlWithProtocol: function (url) {
        try {
            if (url) {
                var prefix_ = '';
                if (url.indexOf('http://') !== -1) {
                    prefix_ = 'http://'
                } else if (url.indexOf('https://') !== -1) {
                    prefix_ = 'https://'
                }
                return prefix_ + olFunctions.GetDomainFromUrl(url);
            }
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('GetDomainFromUrlWithProtocol', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    CompareTabUrl: function (url) {
        try {
            //ignoring relative urls, deleting everything from the # till the end Firefox fix
            inputUrl_ = String(url).toLowerCase().substring(0, 2000);
            sourceUrl_ = String(document.URL).toLowerCase().substring(0, 2000);

            //removing * from start of input url
            if (inputUrl_.indexOf('*') == 0) {
                inputUrl_ = inputUrl_.substring(1, inputUrl_.length);
            }
            //removing hashtag info from urls
            if (inputUrl_.indexOf('#') != -1) {
                inputUrl_ = inputUrl_.substring(0, inputUrl_.indexOf('#') - 1);
            }
            if (sourceUrl_.indexOf('#') != -1) {
                sourceUrl_ = sourceUrl_.substring(0, sourceUrl_.indexOf('#') - 1);
            }

            return (inputUrl_ == 'null' || inputUrl_ == sourceUrl_ || inputUrl_.indexOf(sourceUrl_) !== -1 || sourceUrl_.indexOf(inputUrl_) !== -1);
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('CompareTabUrl', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    CheckWebRuleUrls: function (urls) {
        try {
            var documentUrl_ = String(document.URL).toLowerCase().substring(0, 2000);
            var retValue = false;
            if (documentUrl_) {
                if (urls) {
                    for (var i = 0; i < urls.length; i++) {
                        var url = urls[i].toLowerCase();
                        if (url) {
                            if (url == '*.*') {
                                retValue = true;
                                break;
                            } else {
                                if (url.indexOf('*') == 0) {
                                    url = url.substring(1);
                                }
                                if (documentUrl_.indexOf(url) !== -1) {
                                    retValue = true;
                                    break;
                                }
                            }
                        }
                    }
                }
            }
            return retValue;
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('CheckWebRuleUrls', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    CompareTabUrlForUrlRules: function (url) {
        try {
            inputUrl_ = String(url).toLowerCase().substring(0, 2000);
            sourceUrl_ = String(document.URL).toLowerCase().substring(0, 2000);

            //removing * from start of input url
            if (inputUrl_.indexOf('*') == 0) {
                inputUrl_ = inputUrl_.substring(1, inputUrl_.length);
            }
            return (inputUrl_ == 'null' || inputUrl_ == sourceUrl_ || inputUrl_.indexOf(sourceUrl_) !== -1 || sourceUrl_.indexOf(inputUrl_) !== -1);
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('CompareTabUrlForUrlRules', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    CompareUrls: function (inputUrl, sourceUrl) {
        try {
            //ignoring relative urls, deleting everything from the # till the end Firefox fix
            //but use copy of sources
            var inputUrl_ = inputUrl;
            var sourceUrl_ = sourceUrl;

            if (inputUrl && inputUrl.indexOf('#') != -1) {
                inputUrl_ = inputUrl.substring(0, inputUrl.indexOf('#') - 1);
            }
            if (sourceUrl && sourceUrl.indexOf('#') != -1) {
                sourceUrl_ = sourceUrl.substring(0, sourceUrl.indexOf('#') - 1);
            }
            return inputUrl_ && (inputUrl_ == sourceUrl_ || inputUrl_.indexOf(sourceUrl_) !== -1 || sourceUrl_.indexOf(inputUrl_) !== -1);
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('CompareUrls', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    _checkUrlArray: function (urlArray) {
        try {
            var urlCheck_ = false;
            for (var j = 0; j < urlArray.length; j++) {
                if (olFunctions.CompareTabUrlForUrlRules(urlArray[j])) {
                    urlCheck_ = true;
                    break;
                }
            }
            return urlCheck_;
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('_checkUrlArray', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    CheckUrls: function (matchUrls, notMatchUrls) {
        try {
            var result_;
            if (olFunctions.IsFilledArray(matchUrls)) {
                if (olFunctions._checkUrlArray(matchUrls)) {
                    if (olFunctions.IsFilledArray(notMatchUrls) && olFunctions._checkUrlArray(notMatchUrls)) {
                        result_ = false;
                    } else {
                        result_ = true;
                    }
                } else {
                    result_ = false;
                }
            } else {
                result_ = true;
            }
            return result_;
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('CheckUrls', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    RemoveComments: function (str) {
        try {
            str = ('__' + str + '__').split('');
            var mode = {
                singleQuote: false,
                doubleQuote: false,
                regex: false,
                blockComment: false,
                lineComment: false,
                condComp: false
            };
            for (var i = 0,
                l = str.length; i < l; i++) {

                if (mode.regex) {
                    if (str[i] === '/' && str[i - 1] !== '\\') {
                        mode.regex = false;
                    }
                    continue;
                }

                if (mode.singleQuote) {
                    if (str[i] === "'" && str[i - 1] !== '\\') {
                        mode.singleQuote = false;
                    }
                    continue;
                }

                if (mode.doubleQuote) {
                    if (str[i] === '"' && str[i - 1] !== '\\') {
                        mode.doubleQuote = false;
                    }
                    continue;
                }

                if (mode.blockComment) {
                    if (str[i] === '*' && str[i + 1] === '/') {
                        str[i + 1] = '';
                        mode.blockComment = false;
                    }
                    str[i] = '';
                    continue;
                }

                if (mode.lineComment) {
                    if (str[i + 1] === '\n' || str[i + 1] === '\r') {
                        mode.lineComment = false;
                    }
                    str[i] = '';
                    continue;
                }

                if (mode.condComp) {
                    if (str[i - 2] === '@' && str[i - 1] === '*' && str[i] === '/') {
                        mode.condComp = false;
                    }
                    continue;
                }

                mode.doubleQuote = str[i] === '"';
                mode.singleQuote = str[i] === "'";

                if (str[i] === '/') {

                    if (str[i + 1] === '*' && str[i + 2] === '@') {
                        mode.condComp = true;
                        continue;
                    }
                    if (str[i + 1] === '*') {
                        str[i] = '';
                        mode.blockComment = true;
                        continue;
                    }
                    if (str[i + 1] === '/') {
                        str[i] = '';
                        mode.lineComment = true;
                        continue;
                    }
                    mode.regex = true;
                }
            }
            return str.join('').slice(2, -2);
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('RemoveComments', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    Clone: function (obj) {
        try {
            if (null == obj || "object" != typeof obj)
                return obj;
            var copy_ = obj.constructor();
            for (var attr_ in obj) {
                if (obj.hasOwnProperty(attr_))
                    copy_[attr_] = obj[attr_];
            }
            return copy_;
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('Clone', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    PPA_PushItem: function (grabTextItem, featuresElement, featuresArray) {
        try {
            //if text item can be found push it to features element item array
            var label_ = grabTextItem.Label;
            var textFound_ = null;

            //GetTextBetween
            if (grabTextItem.GetTextBetween) {
                var startWord_ = grabTextItem.GetTextBetween.Start;
                var endWord_ = grabTextItem.GetTextBetween.End;

                for (var i = 0; i < olInjection.ElementValue._internal.Documents.length; i++) {
                    //var text_ = $(olInjection.ElementValue._internal.Documents[i]).text();
                    var text_ = olInjection.ElementValue._internal.Documents[i].textContent;
                    text_ = text_.replace(/\n/g, "");
                    textFound_ = olFunctions.GetTextBetween(text_, startWord_, endWord_);
                    if (textFound_) {
                        break;
                    }
                }
            }
            //ElementSearch
            if (grabTextItem.ElementSearch) {
                for (var i = 0; i < olInjection.ElementValue._internal.Documents.length; i++) {
                    textFound_ = olFunctions.ElementSearch(grabTextItem.ElementSearch, olInjection.ElementValue._internal.Documents[i])[1];
                    if (textFound_) {
                        break;
                    }
                }
            }
            //Finally
            if (textFound_) {
                if (grabTextItem.Transform) {
                    if (grabTextItem.Transform.TruncateSpecified) {
                        textFound_ = olFunctions.TransformTruncate(textFound_, grabTextItem.Transform.Truncate);
                    }
                    if (olFunctions.IsFilledArray(grabTextItem.Transform.Replace)) {
                        textFound_ = olFunctions.TransformReplace(textFound_, grabTextItem.Transform.Replace);
                    }
                }

                var itemElement_ = {
                    Label: label_,
                    Value: textFound_
                };

                if (grabTextItem.IsUniqueSpecified) {
                    itemElement_.IsUniqueSpecified = true;
                    itemElement_.IsUnique = grabTextItem.IsUnique;
                } else {
                    itemElement_.IsUniqueSpecified = false;
                    itemElement_.IsUnique = false;
                }

                if (grabTextItem.IsCostSpecified) {
                    itemElement_.IsCostSpecified = true;
                    itemElement_.IsCost = grabTextItem.IsCost;
                } else {
                    itemElement_.IsCostSpecified = false;
                    itemElement_.IsCost = false;
                }

                //Not sending items that are already sent by other features
                if (olFunctions.IsFilledArray(featuresArray)) {
                    var duplicateItemElement_ = ol$Functions.grep(featuresArray, function (member) {
                        for (var i = 0; i < member.Item.length; i++) {
                            return (member.Item[i].Label == itemElement_.Label && member.Item[i].Value == itemElement_.Value);
                        }
                        return false;
                    });

                    if (duplicateItemElement_.length == 0) {
                        featuresElement.Item.push(itemElement_);
                    }
                } else {
                    featuresElement.Item.push(itemElement_);
                }
            }
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('PPA_PushItem', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    GetTextBetween: function (strToParse, strStart, strFinish) {
        try {
            strStart = strStart.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
            strFinish = strFinish.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
            var result_ = '';
            var str = strToParse.match(strStart + '(.*?)' + strFinish);

            if (olFunctions.IsFilledArray(str)) {
                var result_ = ol$Functions.trim(str[1]);
            }
            return result_;
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('GetTextBetween', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    TransformTruncate: function (text, truncateLength) {
        try {
            return text.substring(0, text.length - truncateLength);
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('TransformTruncate', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    TransformReplace: function (text, replaceObjArray) {
        try {
            var textResult_ = text;
            for (var i = 0; i < replaceObjArray.length; i++) {
                textResult_ = textResult_.replace(new RegExp(String(replaceObjArray[i].Find), 'g'), String(replaceObjArray[i].ReplaceWith));
            }
            return textResult_;
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('TransformReplace', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    ElementSearch: function (elementSearch, container) {
        try {
            if (typeof (container) === 'undefined')
                container = document;

            var elementsArray_ = [];
            var tmpElementsArray_ = [];
            var resultingArray_ = [];
            var type_;
            var typeValue_;

            //Determine search type for the element
            if (elementSearch.Element.Id) {
                type_ = 'Id';
            } else {
                if (elementSearch.Element.Name) {
                    type_ = 'Name';
                } else {
                    if (elementSearch.Element.TagName) {
                        type_ = 'Tag';
                    } else if (elementSearch.Element.IndexSpecified) {
                        type_ = 'Index';
                    }
                }
            }

            switch (type_) {
                case 'Id':
                    typeValue_ = elementSearch.Element.Id;
                    tmpElementsArray_ = olInjection.ElementValue._internal._getElementsByAttributeValue(type_, typeValue_, container);
                    break;
                case 'Name':
                    typeValue_ = elementSearch.Element.Name;
                    tmpElementsArray_ = olInjection.ElementValue._internal._getElementsByAttributeValue(type_, typeValue_, container);
                    break;
                case 'Tag':
                    var filterTagArray_ = false;
                    typeValue_ = elementSearch.Element.TagName;
                    tmpElementsArray_ = container.getElementsByTagName(typeValue_);
                    var tmpElementsArrayFilteredByAttributes_ = [];
                    if (olFunctions.IsFilledArray(elementSearch.Element.Attributes)) {
                        for (var i = 0; i < elementSearch.Element.Attributes.length; i++) {
                            var currentAttribute_ = elementSearch.Element.Attributes[i];
                            if (currentAttribute_.IsMatchSpecified && currentAttribute_.IsMatch) {
                                filterTagArray_ = true;
                                tmpElementsArrayFilteredByAttributes_ = $.merge(tmpElementsArrayFilteredByAttributes_, olInjection.ElementValue._internal._getElementsByAttributeValue(currentAttribute_.Name, currentAttribute_.Value, container));
                            }
                        }

                        if (filterTagArray_) {
                            tmpElementsArray_ = ol$Functions.grep(tmpElementsArray_, function (member) {
                                for (var j = 0; j < tmpElementsArrayFilteredByAttributes_.length; j++) {
                                    if (member == tmpElementsArrayFilteredByAttributes_[j]) {
                                        return true;
                                    };
                                };
                                return false;
                            });
                        }
                    }
                    break;
                case 'Index':
                    var allElements_ = [];
                    allElements_ = olInjection.ElementValue._internal._getElementsByAttributeValue(null, null, container);
                    tmpElementsArray_ = allElements_[elementSearch.Element.Index - 1];
                    break;
                default:
                    olFunctions.Alert(olOptions.Errors(), 'Unknown type value!!!!');
            }

            if (olFunctions.IsFilledArray(elementSearch.FindText)) {
                for (var i = 0; i < tmpElementsArray_.length; i++) {
                    var elements_ = tmpElementsArray_[i];
                    for (var j = 0; j < elementSearch.FindText.length; j++) {
                        elements_ = elements_.filter(':contains("' + elementSearch.FindText[j] + '")');
                    }
                    for (var k = 0; k < elements_.length; k++) {
                        elementsArray_.push(elements_[k]);
                    }
                }
            } else {
                for (var j = 0; j < tmpElementsArray_.length; j++) {
                    elementsArray_.push(tmpElementsArray_[j]);
                }
            }

            var result_ = '';

            if (olFunctions.IsFilledArray(elementsArray_)) {
                if (elementSearch.Element.InstanceSpecified) {
                    resultingArray_.push(elementsArray_[elementSearch.Element.Instance - 1]);
                } else {
                    resultingArray_ = elementsArray_;
                }
                switch (elementSearch.GrabFrom) {
                    case 0:
                        result_ = $(resultingArray_[0]).text();
                        break;
                    case 1:
                        result_ = $(resultingArray_[0]).val();
                        break;
                    case 2:
                        result_ = olFunctions.ElementSearch(elementSearch.InnerNode, resultingArray_[0])[1];
                        break;
                    case 3:
                        for (var i = 0; i < elementSearch.Element.Attributes.length; i++) {
                            var currentAttribute_ = elementSearch.Element.Attributes[i];
                            if (!(currentAttribute_.IsMatchSpecified && currentAttribute_.IsMatch)) {
                                var text_ = resultingArray_[0].getAttribute(currentAttribute_.Name);
                                if (text_) {
                                    result_ = result_ + text_;
                                }
                            }
                        }
                        break;
                    default:
                        olFunctions.Alert(olOptions.Errors(), 'Unknown GrabFrom value!!!!');
                }
            }

            result_ = ol$Functions.trim(result_);
            return [resultingArray_, result_];
        } catch (e) {
            var alertErrorContent_ = new olFunctions.AlertContent('ElementSearch', e.message + '\n' + e.stack);
            olFunctions.Alert(olOptions.Errors(), alertErrorContent_, 'error');
        }
    },
    GetPropertyValue: function (object, dataToRetrieve) {
        dataToRetrieve.split('.').forEach(function (token) {
            if (object) object = object[token];
        });
        return object;
    }
};